/*
   DCX2PCX.prg

   Cut a DCX file into individual PCX file.	

   Copyright 1995 by Extrasensory Software.  Permission granted to
   use and distribute these functions freely in source, object, or
   executable form as long as this copyright notice is not removed
   from the source.

   18 October, 1995
  	
   Parameter : the name of the DCX file to convert.	

   Returns : an array of created PCX filenames.	

*/

function dcx2pcx( filename )
   local in := fopen( filename )
   local out
   local pointers := {}
   local ptrbuffer := space(4)
   local buffer := space(2048)
   local i
   local bytesread
  	local outfiles := {}

  	// Make sure we can open the file
   fread( in, @ptrbuffer, 4 )
   if in < 0
      alert( "I can't find "+filename, {"OK"} )
  	   // return an empty PCX list
      return {}
   endif

  	// check for a DCX signature 	
   if bin2l( ptrBuffer ) <> 987654321
      alert( "This does not seem to be a DCX file", {"OK"} )
  	   // return an empty PCX list
      return {}
   endif

  	// delete the extention so we can create new files. 
   if "." $ filename
      filename := subs( filename, 1, at( ".", filename ) )
   else
     	filename += "."
   endif

  	// get the list of pointers from the DCX file header
   do while bin2l( ptrBuffer ) > 0
      fread( in, @ptrbuffer, 4 )
      aadd( pointers, bin2l( ptrBuffer ) )
   enddo

   // put the file length in the last pointer position
   pointers[len(pointers)] := fseek( in, 0, 2 )

   // copy out the individual PCX files
   for i := 1 to len( pointers ) - 1
     	
     	// move to the start of the next file
      fseek( in, pointers[i], 0 )

     	// create the output file	
      out := fcreate( filename + padl( alltrim( str( i ) ), 3, "0" ) )
     	aadd( outfiles, filename + padl( alltrim( str( i ) ), 3, "0" ) )

     	// copy the input file to the output file	
      do while fseek( in, 0, 1 ) < pointers[ i + 1 ]
        	bytesread := fread( in, @buffer, 2048 )
         fwrite( out, buffer, bytesread )
      enddo

      // set the output file to the correct length
      fseek( out, pointers[i+1] - pointers[i], 0 )
      fwrite( out, "" )

     	// close the output file	
      fclose( out )
   next

  	// close the input file
   fclose( in )

  	// return the list of created PCX files.	
   return outfiles



